-- Shows the compass at the bottom of the screen to the left of the speed hud.
-- Right Alt + Z - Compass on/off

-- V1.00
-- First Version

hudCompass = {
	['version'] = 1.00,
	['active'] = true,
	['setOnce'] = false,
	['setOnceActive'] = false,
	['moddir'] = g_currentModDirectory,
	['textureFilename'] = 'hudCompass_1024.dds',
	['compassPosX'] = 0,
	['compassPosY'] = 0,
	['compassWidth'] = 0,
	['compassHeight'] = 0,
	['targetAspectRatio'] = 16/9,
	['sizeRatio'] = 1,
	['baseHudWidth'] = 0,
	['baseHudX'] = 0
};

local aspectRatioRatio = g_screenAspectRatio / hudCompass.targetAspectRatio;
local baseUVsPx = { 0,142.5, 110,0 };
local mainUVsPx = { 110,103, 213,0 };
local bgUVsPx = { 213,187.5, 333,0 };

if g_screenWidth > 1920 then
	hudCompass.sizeRatio = 1920 / g_screenWidth;
end;

function hudCompass:pxToNormal(px, dimension)
	local ret;
	if dimension == 'x' then
		ret = (px / 1920) * hudCompass.sizeRatio;
	else
		ret = (px / 1080) * hudCompass.sizeRatio * aspectRatioRatio;
	end;

	return ret;
end;

function hudCompass:setOverlayUVsPx(overlay, UVs, textureSizeX, textureSizeY)
	if overlay.overlayId and overlay.currentUVs == nil or overlay.currentUVs ~= UVs then
		local leftX, bottomY, rightX, topY = unpack(UVs);

		local fromTop = false;
		if topY < bottomY then
			fromTop = true;
		end;
		local leftXNormal = leftX / textureSizeX;
		local rightXNormal = rightX / textureSizeX;
		local bottomYNormal = bottomY / textureSizeY;
		local topYNormal = topY / textureSizeY;
		if fromTop then
			bottomYNormal = 1 - bottomYNormal;
			topYNormal = 1 - topYNormal;
		end;
		setOverlayUVs(overlay.overlayId, leftXNormal,bottomYNormal, leftXNormal,topYNormal, rightXNormal,bottomYNormal, rightXNormal,topYNormal);
		overlay.currentUVs = UVs;
	end;
end;

function hudCompass:loadMap(name)
	self.baseHudWidth = g_currentMission.vehicleHudBg.width;
	self.baseHudX = g_currentMission.vehicleHudBg.x;
end;

function hudCompass:deleteMap()
end;

function hudCompass:mouseEvent(posX, posY, isDown, isUp, button)
end;

function hudCompass:keyEvent(unicode, sym, modifier, isDown)
end;

function hudCompass:update(dt)
	if InputBinding.hasEvent(InputBinding.HUDCOMPASS_TOGGLE) then
		if self.active then
			self.active = false;
		else
			self.active = true;
		end
	end
end;

function hudCompass:draw()
	if self.active then
		if self.setOnceActive == false then
			self.setOnceActive = true;
			g_currentMission.vehicleHudBg.width = g_currentMission.vehicleHudBg.width + hudCompass:pxToNormal(110, 'x') + (0.003 / g_screenAspectRatio);
			g_currentMission.vehicleHudBg.x = g_currentMission.vehicleHudBg.x - hudCompass:pxToNormal(110, 'x') - (0.003 / g_screenAspectRatio);
		end
		
		local x, y, z = nil;
		local direction = 0;
		
		if g_currentMission.controlledVehicle ~= nil then -- Controlling vehicle
			x, y, z = localDirectionToWorld(g_currentMission.controlledVehicle.rootNode, 0, 0, 1);
			local length = Utils.vector2Length(x, z);
			if (length ~= 0.0) then
				direction = (math.deg(math.atan2(z / length, x / length)) + 90.0) % 360.0; -- Rotate clockwise, so north=0, east=90, south=180, west=270.
			end
		elseif g_currentMission.player ~= nil then -- On foot
			x, y, z = localDirectionToWorld(g_currentMission.player.lightNode, 0, 0, 1);
			local length = Utils.vector2Length(x, z);
			if (length ~= 0.0) then
				direction = (math.deg(math.atan2(z / length, x / length)) - 90.0) % 360.0; -- Rotate counterclockwise, so north=0, east=90, south=180, west=270.
			end
		end
		
		if g_currentMission.controlledVehicle ~= nil then -- Controlling vehicle
			self.compassPosX = g_currentMission.vehicleSpeedBg.x - hudCompass:pxToNormal(110, 'x') - (0.003 / g_screenAspectRatio);
			self.compassPosY = g_currentMission.vehicleSpeedBg.y;
			self.compassWidth = hudCompass:pxToNormal(110, 'x');
			self.compassHeight = hudCompass:pxToNormal(142.5, 'y');
			self.compassMainWidth = hudCompass:pxToNormal(103, 'x');
			self.compassMainHeight = hudCompass:pxToNormal(103, 'y');
			
			self.compassBase = Overlay:new('compassBase', Utils.getFilename(self.textureFilename, self.moddir), self.compassPosX, self.compassPosY, self.compassWidth, self.compassHeight);
			hudCompass:setOverlayUVsPx(self.compassBase, baseUVsPx, 1024, 1024);
			self.compassBase:render();

			self.compassMain = Overlay:new('compassMain', Utils.getFilename(self.textureFilename, self.moddir), self.compassPosX + hudCompass:pxToNormal(3.5, 'x'), self.compassPosY + hudCompass:pxToNormal(39.5, 'y'), self.compassMainWidth, self.compassMainHeight);
			self.compassMain:setRotation(direction * (3.142 / 180), self.compassMainWidth / 2, self.compassMainHeight / 2);
			hudCompass:setOverlayUVsPx(self.compassMain, mainUVsPx, 1024, 1024);
			self.compassMain:render();

			local directionText = string.format('%5.1f', direction) .. '°';
			local directionTextWidth = getTextWidth(0.016, directionText);
			renderText(self.compassPosX + self.compassWidth / 2 - directionTextWidth / 2, self.compassPosY + 0.01, 0.016, directionText);
			
		elseif g_currentMission.player ~= nil then -- On foot
			self.compassPosX = g_currentMission.vehicleHudBg.x + g_currentMission.vehicleHudBg.width - hudCompass:pxToNormal(110, 'x');
			self.compassPosY = g_currentMission.vehicleSpeedBg.y;
			self.compassWidth = hudCompass:pxToNormal(110, 'x');
			self.compassHeight = hudCompass:pxToNormal(142.5, 'y');
			self.compassMainWidth = hudCompass:pxToNormal(103, 'x');
			self.compassMainHeight = hudCompass:pxToNormal(103, 'y');
			self.compassBgPosX = g_currentMission.vehicleHudBg.x + g_currentMission.vehicleHudBg.width - hudCompass:pxToNormal(126.5, 'x');
			self.compassBgWidth = hudCompass:pxToNormal(132.5, 'x');
			self.compassBgHeight = hudCompass:pxToNormal(187.5, 'y');
			
			self.compassBg = Overlay:new('compassBg', Utils.getFilename(self.textureFilename, self.moddir), self.compassBgPosX, g_currentMission.vehicleHudBg.y, self.compassBgWidth, self.compassBgHeight);
			hudCompass:setOverlayUVsPx(self.compassBg, bgUVsPx, 1024, 1024);
			self.compassBg:setColor(g_currentMission.vehicleHudBg.r, g_currentMission.vehicleHudBg.g, g_currentMission.vehicleHudBg.b, 1);
			self.compassBg:render();
			
			self.compassBase = Overlay:new('compassBase', Utils.getFilename(self.textureFilename, self.moddir), self.compassPosX, self.compassPosY, self.compassWidth, self.compassHeight);
			hudCompass:setOverlayUVsPx(self.compassBase, baseUVsPx, 1024, 1024);
			self.compassBase:render();

			self.compassMain = Overlay:new('compassMain', Utils.getFilename(self.textureFilename, self.moddir), self.compassPosX + hudCompass:pxToNormal(3.5, 'x'), self.compassPosY + hudCompass:pxToNormal(39.5, 'y'), self.compassMainWidth, self.compassMainHeight);
			self.compassMain:setRotation(direction * (3.142 / 180), self.compassMainWidth / 2, self.compassMainHeight / 2);
			hudCompass:setOverlayUVsPx(self.compassMain, mainUVsPx, 1024, 1024);
			self.compassMain:render();

			local directionText = string.format('%5.1f', direction) .. '°';
			local directionTextWidth = getTextWidth(0.016, directionText);
			renderText(self.compassPosX + self.compassWidth / 2 - directionTextWidth / 2, self.compassPosY + 0.01, 0.016, directionText);

		end
	else
		if self.setOnce == false then
			self.setOnce = true;
			g_currentMission.vehicleHudBg.width = self.baseHudWidth;
			g_currentMission.vehicleHudBg.x = self.baseHudX;
		end
	end
end;

addModEventListener(hudCompass);

print(string.format('[RokMods] Script loaded: hudCompass.lua (v%.2f)', hudCompass.version));